<?php

namespace Drupal\content_model_documentation;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleExtensionList;

/**
 * Service providing information on the Documentable Modules.
 */
class DocumentableModules {

  /**
   * Module configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Config factory service.
   * @param \Drupal\Core\Extension\ModuleExtensionList $extension_list_module
   *   The module extension list.
   */
  public function __construct(ConfigFactoryInterface $configFactory, ModuleExtensionList $extension_list_module) {
    $this->config = $configFactory->get('content_model_documentation.settings');
    $this->moduleExtensionList = $extension_list_module;
  }

  /**
   * Gets an array of all modules.
   *
   * Disabled and enabled modules are included because we may want to document
   * a module that we keep around even when disabled.
   *
   * @return array
   *   Modules listed for documentation.
   */
  public function getDocumentableModulesSelectList(): array {
    $module_list = [];
    // Elements should follow the pattern 'machine_name' => 'name'.
    $extensions = $this->moduleExtensionList->getList();
    foreach ($extensions as $extension) {
      if ($extension->info['type'] === 'module') {
        $project = (!empty($extension->info['project'])) ? $extension->info['project'] : $extension->origin;
        $module_name = $extension->info['name'];
        if ((isset($extension->info['project']) && $extension->getName() !== $extension->info['project']) || $project === 'core') {
          // This is likely a submodule.
          $module_list["module.{$project}.{$extension->getName()}"] = "module.{$project}.{$extension->getName()}";
        }
        elseif (!empty($project)) {
          // The project is the module.
          $module_list["module.{$project}"] = "module.{$project}";
        }
      }
    }

    $module_list = $this->removeModules($module_list);
    asort($module_list, SORT_NATURAL);

    return $module_list;
  }

  /**
   * Get the info for reporting on enabled modules.
   *
   * @return array
   *   A keyed array of module properties.
   */
  public function getEnabledReporting(): array {
    // @todo Flesh this out.
    // We never return disabled modules because that would be a security risk
    // since the report we show may not require the same user as module admin.
    return [];
  }

  /**
   * Removes modules that should not be revealed to non-admins (devel, etc).
   *
   * @param array $documentable_modules
   *   An array of key value pairs to have modules removed from.
   *
   * @return array
   *   An array of key value pairs for the module that are documentable.
   */
  protected function removeModules(array $documentable_modules): array {
    $remove_these = [
      'module.devel' => NULL,
    ];

    return array_diff_key($documentable_modules, $remove_these);
  }

}
